@@ -15,117 +15,82 @@ from .GraphicsObject import GraphicsObject
 __all__ = ['PlotCurveItem']
 
 
-def have_native_drawlines_array():
-    size = 10
-    line = QtCore.QLineF(0, 0, size, size)
-    qimg = QtGui.QImage(size, size, QtGui.QImage.Format.Format_RGB32)
-    qimg.fill(QtCore.Qt.GlobalColor.transparent)
-    painter = QtGui.QPainter(qimg)
-    painter.setPen(QtCore.Qt.GlobalColor.white)
+def arrayToLineSegments(x, y, connect, finiteCheck, out=None):
+    if out is None:
+        out = Qt.internals.PrimitiveArray(QtCore.QLineF, 4)
 
-    try:
-        painter.drawLines(line, 1)
-    except TypeError:
-        success = False
-    else:
-        success = True
-    finally:
-        painter.end()
+    # analogue of arrayToQPath taking the same parameters
+    if len(x) < 2:
+        out.resize(0)
+        return out
 
-    return success
+    connect_array = None
+    if isinstance(connect, np.ndarray):
+        # the last element is not used
+        connect_array, connect = np.asarray(connect[:-1], dtype=bool), 'array'
 
-_have_native_drawlines_array = Qt.QT_LIB.startswith('PySide') and have_native_drawlines_array()
+    all_finite = True
+    if finiteCheck or connect == 'finite':
+        mask = np.isfinite(x) & np.isfinite(y)
+        all_finite = np.all(mask)
 
+    if connect == 'all':
+        if not all_finite:
+            # remove non-finite points, if any
+            x = x[mask]
+            y = y[mask]
 
-class LineSegments:
-    def __init__(self):
-        use_array = None
-
-        # "use_native_drawlines" is pending the following issue and code review
-        # https://bugreports.qt.io/projects/PYSIDE/issues/PYSIDE-1924
-        # https://codereview.qt-project.org/c/pyside/pyside-setup/+/415702
-        self.use_native_drawlines = Qt.QT_LIB.startswith('PySide') and _have_native_drawlines_array
-        if self.use_native_drawlines:
-            use_array = True
-
-        self.array = Qt.internals.PrimitiveArray(QtCore.QLineF, 4, use_array=use_array)
-
-    def get(self, size):
-        self.array.resize(size)
-        return self.array.instances(), self.array.ndarray()
-
-    def arrayToLineSegments(self, x, y, connect, finiteCheck):
-        # analogue of arrayToQPath taking the same parameters
-        if len(x) < 2:
-            return [],
-
-        connect_array = None
-        if isinstance(connect, np.ndarray):
-            # the last element is not used
-            connect_array, connect = np.asarray(connect[:-1], dtype=bool), 'array'
-
-        all_finite = True
-        if finiteCheck or connect == 'finite':
-            mask = np.isfinite(x) & np.isfinite(y)
-            all_finite = np.all(mask)
-
-        if connect == 'all':
-            if not all_finite:
-                # remove non-finite points, if any
-                x = x[mask]
-                y = y[mask]
-
-        elif connect == 'finite':
-            if all_finite:
-                connect = 'all'
-            else:
-                # each non-finite point affects the segment before and after
-                connect_array = mask[:-1] & mask[1:]
-
-        elif connect in ['pairs', 'array']:
-            if not all_finite:
-                # replicate the behavior of arrayToQPath
-                backfill_idx = fn._compute_backfill_indices(mask)
-                x = x[backfill_idx]
-                y = y[backfill_idx]
-
-        segs = []
-        nsegs = 0
-
-        if connect == 'all':
-            nsegs = len(x) - 1
-            if nsegs:
-                segs, memory = self.get(nsegs)
-                memory[:, 0] = x[:-1]
-                memory[:, 2] = x[1:]
-                memory[:, 1] = y[:-1]
-                memory[:, 3] = y[1:]
-
-        elif connect == 'pairs':
-            nsegs = len(x) // 2
-            if nsegs:
-                segs, memory = self.get(nsegs)
-                memory = memory.reshape((-1, 2))
-                memory[:, 0] = x[:nsegs * 2]
-                memory[:, 1] = y[:nsegs * 2]
-
-        elif connect_array is not None:
-            # the following are handled here
-            # - 'array'
-            # - 'finite' with non-finite elements
-            nsegs = np.count_nonzero(connect_array)
-            if nsegs:
-                segs, memory = self.get(nsegs)
-                memory[:, 0] = x[:-1][connect_array]
-                memory[:, 2] = x[1:][connect_array]
-                memory[:, 1] = y[:-1][connect_array]
-                memory[:, 3] = y[1:][connect_array]
-
-        if nsegs and self.use_native_drawlines:
-            return segs, nsegs
+    elif connect == 'finite':
+        if all_finite:
+            connect = 'all'
         else:
-            return segs,
+            # each non-finite point affects the segment before and after
+            connect_array = mask[:-1] & mask[1:]
 
+    elif connect in ['pairs', 'array']:
+        if not all_finite:
+            # replicate the behavior of arrayToQPath
+            backfill_idx = fn._compute_backfill_indices(mask)
+            x = x[backfill_idx]
+            y = y[backfill_idx]
+
+    if connect == 'all':
+        nsegs = len(x) - 1
+        out.resize(nsegs)
+        if nsegs:
+            memory = out.ndarray()
+            memory[:, 0] = x[:-1]
+            memory[:, 2] = x[1:]
+            memory[:, 1] = y[:-1]
+            memory[:, 3] = y[1:]
+
+    elif connect == 'pairs':
+        nsegs = len(x) // 2
+        out.resize(nsegs)
+        if nsegs:
+            memory = out.ndarray()
+            memory = memory.reshape((-1, 2))
+            memory[:, 0] = x[:nsegs * 2]
+            memory[:, 1] = y[:nsegs * 2]
+
+    elif connect_array is not None:
+        # the following are handled here
+        # - 'array'
+        # - 'finite' with non-finite elements
+        nsegs = np.count_nonzero(connect_array)
+        out.resize(nsegs)
+        if nsegs:
+            memory = out.ndarray()
+            memory[:, 0] = x[:-1][connect_array]
+            memory[:, 2] = x[1:][connect_array]
+            memory[:, 1] = y[:-1][connect_array]
+            memory[:, 3] = y[1:][connect_array]
+
+    else:
+        nsegs = 0
+        out.resize(nsegs)
+
+    return out
 
 class PlotCurveItem(GraphicsObject):
     """
@@ -573,7 +538,7 @@ class PlotCurveItem(GraphicsObject):
         self.fillPath = None
         self._fillPathList = None
         self._mouseShape = None
-        self._renderSegmentList = None
+        self._lineSegmentsRendered = False
 
         if 'name' in kargs:
             self.opts['name'] = kargs['name']
@@ -703,10 +668,7 @@ class PlotCurveItem(GraphicsObject):
         )
 
     def _getLineSegments(self):
-        if not hasattr(self, '_lineSegments'):
-            self._lineSegments = LineSegments()
-
-        if self._renderSegmentList is None:
+        if not self._lineSegmentsRendered:
             x, y = self.getData()
             if self.opts['stepMode']:
                 x, y = self._generateStepModeData(
@@ -716,14 +678,17 @@ class PlotCurveItem(GraphicsObject):
                     baseline=self.opts['fillLevel']
                 )
 
-            self._renderSegmentList = self._lineSegments.arrayToLineSegments(
+            self._lineSegments = arrayToLineSegments(
                 x,
                 y,
                 connect=self.opts['connect'],
-                finiteCheck=not self.opts['skipFiniteCheck']
+                finiteCheck=not self.opts['skipFiniteCheck'],
+                out=self._lineSegments
             )
 
-        return self._renderSegmentList
+            self._lineSegmentsRendered = True
+
+        return self._lineSegments.drawargs()
 
     def _getClosingSegments(self):
         # this is only used for fillOutline
@@ -988,7 +953,8 @@ class PlotCurveItem(GraphicsObject):
     def clear(self):
         self.xData = None  ## raw values
         self.yData = None
-        self._renderSegmentList = None
+        self._lineSegments = None
+        self._lineSegmentsRendered = False
         self.path = None
         self.fillPath = None
         self._fillPathList = None
