@@ -1,13 +1,12 @@
-# -*- coding: utf-8 -*-
 """
 PlotWidget.py -  Convenience class--GraphicsView widget displaying a single PlotItem
 Copyright 2010  Luke Campagnola
 Distributed under MIT/X11 license. See license.txt for more information.
 """
 
-from ..Qt import QtCore, QtGui
-from .GraphicsView import *
-from ..graphicsItems.PlotItem import *
+from ..graphicsItems.PlotItem import PlotItem
+from ..Qt import QtCore, QtWidgets
+from .GraphicsView import GraphicsView
 
 __all__ = ['PlotWidget']
 class PlotWidget(GraphicsView):
@@ -50,7 +49,7 @@ class PlotWidget(GraphicsView):
         and all others are passed
         to :func:`PlotItem.__init__() <pyqtgraph.PlotItem.__init__>`."""
         GraphicsView.__init__(self, parent, background=background)
-        self.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
+        self.setSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Expanding)
         self.enableMouse(False)
         if plotItem is None:
             self.plotItem = PlotItem(**kargs)
